use std::collections::HashMap;

use rocket::{
	fairing::Fairing,
	outcome::try_outcome,
	request::{FromRequest, Outcome},
	serde::Serialize,
	Request,
};
use rocket_dyn_templates::{tera::try_get_value, Template};
use serde_json::{to_value, Value};

use crate::{auth::session::Session, ids};

pub fn init_templates() -> impl Fairing {
	Template::custom(|engines| {
		engines
			.tera
			.register_filter("pretty_id", |v: &Value, _: &HashMap<String, Value>| {
				let value = try_get_value!("pretty_id", "value", i32, v);
				Ok(to_value(ids::id_db_to_public(value)).unwrap())
			});
	})
}

/// A bundle of all the state all pages may make use of.
///
/// Implements [`FromRequest`] to provide easy access to it as a guard.
/// This guard never fails.
#[derive(Debug, Serialize)]
#[serde(crate = "rocket::serde")]
pub struct CommonTemplateState {
	/// true if the user is logged in (doesn't check the DB, instead uses [`Session`])
	pub logged_in: bool,
	/// feature flag - disables the UI for it since it's not implemeted yet.
	pub forgot_password_implemented: bool,
}

#[rocket::async_trait]
impl<'r> FromRequest<'r> for CommonTemplateState {
	type Error = ();

	async fn from_request(request: &'r Request<'_>) -> Outcome<CommonTemplateState, ()> {
		let session_state = try_outcome!(request.guard::<Session>().await);

		Outcome::Success(CommonTemplateState {
			logged_in: session_state.0.is_some(),
			forgot_password_implemented: false,
		})
	}
}
